﻿namespace Viss.DIT.WS.Samples.ACKStatusChange
{
    using System;
    using System.ServiceModel;
    using System.ServiceModel.Description;
    using System.Xml;
    using ProxyToService;
    using ProxyToService.DitService;
    using System.Linq;
    using Viss.DIT.WS.Samples.ProxyToService.DitConfiguration;

    public class Program
    {
        public static void Main(string[] args)
        {
            string messageId = string.Empty;
            string channelName = "SampleChannel455";
            string channelURN = "URN:IVIS:100001:CHA-" + channelName + "-TYPE-DIK";
            string channelVersionURN = "URN:IVIS:100001:CHA-SampleChannel455-v1-0-TYPE-DIK";

            using (ProxyToService.DitConfiguration.DITConfigurationServiceContractClient client = new ProxyToService.DitConfiguration.DITConfigurationServiceContractClient("ConfigurationService"))
            {
                try
                {
                    ProxyToService.DitConfiguration.ChannelSearchInfoStructure req = new ProxyToService.DitConfiguration.ChannelSearchInfoStructure();
                    req.Channel = new ProxyToService.DitConfiguration.ChannelInfoStructure();
                    req.Channel.ChannelURN = channelURN;
                    req.OnlyHeader = true;
                    ProxyToService.DitConfiguration.ChannelListStructure channels = Helper.InvokeIgnoreException<ChannelListStructure, CommunicationException>(() => client.SearchChannelsPaged(req, ChannelVersionStatusType.testing), "Argument exception: ChannelURN!");
                    ChannelInfoStructure channel = new ChannelInfoStructure();
                    channel.ShortName = channelName;
                    channel.ChannelURN = channelURN;
                    channel.ChannelType = ChannelType.dik;
                    channel.OwnerAuthority = new AuthorityBasicInfoStructure();
                    channel.OwnerAuthority.AuthorityID = "100000";
                    channel.OwnerAuthority.ShortName = "ABC";
                    channel.OwnerAuthority.Name = "ABC Software";
                    channel.ChannelName = channelName;
                    channel.ChannelTypeSpecified = true;

                    bool channelExsists = channels != null && channels.Count > 0;
                    if (!channelExsists)
                    {
                        int channelId = client.InsertChannel(channel);
                        Console.WriteLine("Channel created");
                    }
                    else
                    {
                        Console.WriteLine("Channel found");
                    }

                    ChannelVersionInfoStructure channelVersion = new ChannelVersionInfoStructure();
                    channelVersion.UseReceiverFilter = true;
                    channelVersion.UseReceiverFilterSpecified = true;
                    channelVersion.MessageType = new ProxyToService.DitConfiguration.MessageTypeInfoStructure[2];

                    channelVersion.MessageType[0] = new ProxyToService.DitConfiguration.MessageTypeInfoStructure();
                    channelVersion.MessageType[0].MessageClass = new ProxyToService.DitConfiguration.MessageClassInfoStructure();

                    // 1 Biznesa ziņojums
                    // 2 Transakcijas ziņojums
                    // 3 error
                    channelVersion.MessageType[0].MessageClass.CodeListCodeValue = "1";
                    channelVersion.MessageType[0].XMLSchemaURN = "URN:IVIS:100001:XSD-Person-FullName-v1-0-TYPE-PersonFullName";

                    channelVersion.MessageType[1] = new ProxyToService.DitConfiguration.MessageTypeInfoStructure();
                    channelVersion.MessageType[1].MessageClass = new ProxyToService.DitConfiguration.MessageClassInfoStructure();
                    channelVersion.MessageType[1].MessageClass.CodeListCodeValue = "3";
                    channelVersion.MessageType[1].XMLSchemaURN = "URN:IVIS:100001:XSD-DIT-DITMessage-v1-0-TYPE-DefaultMessageError";
                    channelVersion.ChannelVersionStatus = ChannelVersionStatusType.active;
                    channelVersion.ChannelVersionStatusSpecified = true;

                    channelVersion.Channel = channel;
                    channelVersion.MinorVersionSpecified = true;
                    channelVersion.MinorVersion = 0;
                    channelVersion.MajorVersionSpecified = true;
                    channelVersion.MajorVersion = 1;

                    channelVersion.IsManualAckSpecified = true;
                    channelVersion.IsManualAck = true;

                    ChannelVersionSearchInfoStructure channelVersionSearch = new ChannelVersionSearchInfoStructure();
                    channelVersionSearch.ChannelVersion = new ChannelVersionInfoStructure();
                    channelVersionSearch.ChannelVersion = channelVersion;

                    ChannelVersionListStructure channelVersions = client.SearchChannelVersionsPaged(channelVersionSearch);

                    if (channelVersions.Count == 0)
                    {
                        int channelId = client.InsertChannelVersion(channelVersion);
                        Console.WriteLine("Channel version created");
                    }
                    else
                    {
                        Console.WriteLine("Channel version found");
                    }

                    ChannelSubscriberInfoStructure channelSubscriber = new ChannelSubscriberInfoStructure();
                    channelSubscriber.ChannelVersionURN = channelVersionURN;
                    channelSubscriber.SubscriberAuthority = new AuthorityBasicInfoStructure();
                    channelSubscriber.SubscriberAuthority.AuthorityID = "100001";
                    channelSubscriber.SubscriberAuthority.ShortName = "VRAA";
                    channelSubscriber.SubscriberAuthority.Name = "VRAA";
                    ChannelSubscriberSearchInfoStructure channelSubscriberSearch = new ChannelSubscriberSearchInfoStructure();
                    channelSubscriberSearch.ChannelSubscriber = channelSubscriber;
                    ChannelSubscriberListStructure channelSubscriberList = client.SearchSubscribersToChannelPaged(channelSubscriberSearch);
                    if (channelSubscriberList.Count == 0)
                    {
                        int result = client.InsertSubscriberToChannel(channelSubscriber);
                        Console.WriteLine("Channel subscriber created");
                    }
                    else
                    {
                        Console.WriteLine("Channel subscriber found");
                    }


                }
                catch (CommunicationException e)
                {
                    string mess = e.Message;
                    Console.WriteLine(mess);

                    client.Abort();
                }
                catch (TimeoutException e)
                {
                    string mess = e.Message;
                    Console.WriteLine(mess);

                    client.Abort();
                }
                catch (Exception e)
                {
                    string mess = e.Message;
                    Console.WriteLine(mess);
                    client.Abort();
                    throw;
                }
            }

            using (ProxyToService.DitService.DITMessageServiceContractClient client = new ProxyToService.DitService.DITMessageServiceContractClient("MessageService"))
            {
                try
                {
                    ProxyToService.DitService.MessageSaveStructure2 message = new ProxyToService.DitService.MessageSaveStructure2();
                    message.ChannelVersionURN = channelVersionURN;
                    BussinesMessageType bussinesMessage = new BussinesMessageType();
                    XmlDocument xml = new XmlDocument();
                    xml.Load("../../../data.xml");
                    bussinesMessage.Any = xml.DocumentElement;

                    message.Item = bussinesMessage;
                    message.ReceiverFilter = new ReceiverFilterArrayStructure() { Authority = new string[] { "100001" } };
                    bool resp = client.SendMessage2(message);
                }
                catch (CommunicationException e)
                {
                    string mess = e.Message;
                    Console.WriteLine(mess);

                    client.Abort();
                }
                catch (TimeoutException e)
                {
                    string mess = e.Message;
                    Console.WriteLine(mess);

                    client.Abort();
                }
                catch (Exception e)
                {
                    string mess = e.Message;
                    Console.WriteLine(mess);
                    client.Abort();
                    throw;
                }
            }

            using (ProxyToService.DitService.DITMessageServiceContractClient client = new ProxyToService.DitService.DITMessageServiceContractClient("MessageServiceUser"))
            {
                var applyTo = new Uri("https://ivis.eps.gov.lv/DIT.WebService");
                var factory = client.ChannelFactory;
                var other = factory.Endpoint.Behaviors.Find<ClientCredentials>();
                factory.Endpoint.Behaviors.Remove(other.GetType());
                var item1 = new ApplyToClientCredentials(other) { ApplyTo = applyTo };
                factory.Endpoint.Behaviors.Add(item1);

                try
                {
                    client.ClientCredentials.UserName.UserName = System.Configuration.ConfigurationManager.AppSettings.Get("VraaUserName");
                    client.ClientCredentials.UserName.Password = System.Configuration.ConfigurationManager.AppSettings.Get("VraaPassword");

                    ProxyToService.DitService.MessageRequestStructure messageRequest = new ProxyToService.DitService.MessageRequestStructure();
                    messageRequest.Item = channelVersionURN;
                    messageRequest.ItemElementName = ProxyToService.DitService.ItemChoiceType.ChannelVersionURN;
                    MessageResponseStructure2 resp = client.ReceiveMessage2(messageRequest);
                    messageId = resp.Items?.FirstOrDefault()?.MessageID;
                    client.ConfirmationAck(Guid.Parse(messageId));
                    Console.WriteLine("ReceiveMessage");
                }
                catch (CommunicationException e)
                {
                    string mess = e.Message;
                    Console.WriteLine(mess);

                    client.Abort();
                }
                catch (TimeoutException e)
                {
                    string mess = e.Message;
                    Console.WriteLine(mess);

                    client.Abort();
                }
                catch (Exception e)
                {
                    string mess = e.Message;
                    Console.WriteLine(mess);
                    client.Abort();
                    throw;
                }
            }
            Console.ReadLine();
        }
    }
}
